﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Threading.Tasks;
using EbayDemo.Impl;
using EbayDemo.Models;
using EbayDemo.Services;
using Newtonsoft.Json;
using Xamarin.Forms;

[assembly: Dependency(typeof(EbayService))]
namespace EbayDemo.Impl
{
    public class EbayService : IEbayService
    {
        private static string BaseUrl = "http://open.api.ebay.com/Shopping?version=800";
        private static string AppId = "&appid=OlivierD-Xformsde-PRD-fe6e68035-2135eba6";
        private static string Encode = "&responseencoding=JSON";
        private static string CallName = "&callname=";
        private static string GetCategoriesParam = "GetCategoryInfo&IncludeSelector=ChildCategories";
        private static string GetItemsParam = "FindPopularItems";
        private static string IdParam = "&CategoryID={0}";


        private static string buildURLForCategories(Category parentCategory)
        {
            return BaseUrl + AppId + Encode + CallName + GetCategoriesParam + string.Format(IdParam, parentCategory?.CategoryID ?? "-1");
        }

        private static string buildURLForItems(Category parentCategory)
        {
            return BaseUrl + AppId + Encode + CallName + GetItemsParam + string.Format(IdParam, parentCategory?.CategoryID ?? "-1");
        }

        private static async Task<string> getHttpStringTask(string url)
        {
            using (var client = new HttpClient())
                return await client.GetStringAsync(url);
        }

        public async Task<List<Category>> GetCategories(Category parentCategory)
        {

            var cat = await getHttpStringTask(buildURLForCategories(parentCategory));
            var data = JsonConvert.DeserializeObject<CategoryRootObject>(cat);
            var result = (from c in data.CategoryArray.Category
                          where c.CategoryID != parentCategory?.CategoryID
                          select c).ToList();
            if (result.Count > 0 && string.Compare(result[0].CategoryName, "root", StringComparison.OrdinalIgnoreCase) == 0) result.RemoveAt(0);
            return result;
        }

        public async Task<List<Item>> GetItems(Category parentCategory)
        {
            var items = await getHttpStringTask(buildURLForItems(parentCategory));
            var data = JsonConvert.DeserializeObject<ItemRootObject>(items);
            return data.ItemArray.Item;
        }
    }
}
